package com.caimei.controller.admin.auth;

import com.caimei.service.auth.DownloadService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author Aslee
 * @date 2021/5/14
 */
@Api(tags = "下载API")
@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping("/download")
public class DownloadApi {
    private static final Logger logger = LoggerFactory.getLogger(DownloadApi.class);
    @Value("${caimei.imageDomain}")
    private String imageDomain;
    private final DownloadService downloadService;


    @ApiOperation("下载文件")
    @ApiImplicitParam(name = "ossName",value = "oss名称")
    @GetMapping("/file")
    public void downloadFile(String ossName,String fileName, HttpServletRequest request, HttpServletResponse response) {
        try {
            downloadService.downloadFile(ossName, fileName, request, response);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }


    @ApiOperation("一键下载授权牌/设备二维码/机构二维码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "authUserId", required = true, value = "供应商用户id"),
            @ApiImplicitParam(name = "type", required = true, value = "1授权牌，2设备二维码，3机构二维码")
    })
    @GetMapping("/shop/image")
    public void downloadImage(Integer authUserId, Integer type, HttpServletRequest request, HttpServletResponse response) throws Exception {
        downloadService.downloadShopImage(authUserId, type, request, response);
    }
}
