package com.caimei.service.auth;

import com.caimei.model.ResponseJson;
import com.caimei.model.po.CmBrandAuthPo;
import com.caimei.model.vo.AuthFormVo;
import com.caimei.model.vo.AuthVo;
import com.caimei.model.vo.TemplateVo;
import com.github.pagehelper.PageInfo;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/5/11
 */
public interface AuthService {

    /**
     * 授权列表
     *
     * @param listType         列表类型：1授权列表，2授权审核列表
     * @param authUserId       供应商用户id
     * @param authParty        授权机构
     * @param mobile
     * @param status           上线状态：0已下线，1已上线，2待上线
     * @param auditStatus      审核状态:0审核未通过，1审核通过，2待审核
     * @param lowerAuditStatus 商品信息审核状态:0未完成审核，1已完成审核
     * @param shopAuditStatus  供应商审核状态:0审核未通过，1审核通过，2待审核
     * @param pageNum          第几页
     * @param pageSize         一页多少条
     * @return AuthVo
     */
    ResponseJson<PageInfo<AuthVo>> getAuthList(Integer listType, Integer authUserId, String authParty, String mobile, Integer status, Integer auditStatus, Integer lowerAuditStatus, Integer shopAuditStatus, Integer sendStatus, Integer pageNum, Integer pageSize);

    /**
     * 更新授权状态
     *
     * @param authId 授权id
     * @param status 授权状态:0停用 1启用
     * @return ResponseJson
     */
    ResponseJson updateAuthStatus(Integer authId, Integer status);

    /**
     * 删除授权
     * @param authId    授权id
     * @return  ResponseJson
     */
    ResponseJson deleteAuth(Integer authId);

    /**
     * 添加/编辑授权
     *
     * @param auth       授权机构
     * @param bannerList 轮播图列表
     * @param source     来源：1供应商保存，2机构保存
     * @return ResponseJson
     */
    ResponseJson saveAuth(CmBrandAuthPo auth, List<String> bannerList, boolean importFlag, Integer source);

    /**
     * 审核品牌授权
     *
     * @param authId            授权id
     * @param auditStatus       审核状态
     * @param invalidReason     审核不通过原因
     * @param auditBy           审核人用户id
     * @param source
     * @return ResponseJson
     */
    ResponseJson auditAuth(Integer authId, Integer auditStatus, String invalidReason, Integer auditBy, Integer source);

    /**
     * 导入excel中的授权机构和商品数据
     * @param file          excel表格
     * @param authUserId    供应商用户id
     * @param createBy
     * @return
     */
    ResponseJson importDataByExcel(MultipartFile file, Integer authUserId, Integer createBy);


    /**
     * 导出授权机构和商品数据
     * @param authUserId    供应商用户id
     * @param response
     * @return
     */
    ResponseJson exportDataByExcel(Integer authUserId, HttpServletResponse response);

    /**
     * 授权机构回显数据
     * @param authId    机构id
     * @return
     */
    ResponseJson<AuthFormVo> getAuthFormData(Integer authId);

    /**
     * 机构下拉框列表
     */
    ResponseJson<List<AuthVo>> getAuthSelectList(Integer authUserId);

    /**
     * ldm门店图/备注导入
     * @param authUserId
     * @return
     */
    ResponseJson importLdmImage(Integer authUserId);

    /**
     * 生成机构授权牌
     * @param auth
     * @return
     */
    String generateAuthImage(TemplateVo authTemplate, CmBrandAuthPo auth);

    /**
     * 添加水印
     */
    String addWaterMark(String image,Integer type);
}
