package com.caimei.service.auth.impl;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.caimei.mapper.cmMapper.AuthMapper;
import com.caimei.mapper.cmMapper.AuthProductMapper;
import com.caimei.mapper.cmMapper.ShopMapper;
import com.caimei.model.ResponseJson;
import com.caimei.model.dto.ProductSaveDto;
import com.caimei.model.po.ProductParamPo;
import com.caimei.model.po.ProductPo;
import com.caimei.model.po.ProductTypePo;
import com.caimei.model.vo.*;
import com.caimei.service.auth.AuthProductService;
import com.caimei.service.auth.UploadService;
import com.caimei.utils.Base64Util;
import com.caimei.utils.ImageUtils;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ClassPathResource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.*;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Aslee
 * @date : 2021/5/17
 */
@Slf4j
@Service
public class AuthProductServiceImpl implements AuthProductService {

    @Resource
    private AuthProductMapper authProductMapper;

    @Resource
    private ShopMapper shopMapper;

    @Resource
    private AuthMapper authMapper;

    @Value("${caimei.imageDomain}")
    private String imageDomain;

    private UploadService uploadService;

    @Value("${spring.profiles.active}")
    private String active;

    @Value("${caimei.oldapi}")
    private String wwwServer;

    @Autowired
    public void setUploadService(UploadService uploadService) {
        this.uploadService = uploadService;
    }

    private String endpoint = "https://oss-cn-shenzhen.aliyuncs.com";
    private String accessKeyId = "LTAI4GBL3o4YkWnbKYgf2Xia";
    private String accessKeySecret = "dBjAXqbYiEPP6Ukuk2ZsXQeET7FVkK";
    private String bucketName = "caimei-oss";

    @Override
    public ResponseJson<PageInfo<ProductListVo>> getProductList(Integer listType, Integer authId, String productName,
                                                                String snCode, Integer status, Integer auditStatus,
                                                                Integer shopAuditStatus, Integer pageNum, Integer pageSize) {
        if (null == authId) {
            return ResponseJson.error("参数异常，请输入授权id", null);
        }
        listType = null == listType ? 1 : listType;
        PageHelper.startPage(pageNum, pageSize);
        List<ProductListVo> productList = authProductMapper.getProductList(listType, authId, productName, snCode, status, auditStatus, shopAuditStatus);
        PageInfo<ProductListVo> pageData = new PageInfo<>(productList);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson updateProductStatus(Integer productId, Integer status) {
        if (productId == null) {
            return ResponseJson.error("请输入用户id");
        }
        if (status == null) {
            return ResponseJson.error("请输入要更新的状态值");
        } else if (status != 0 && status != 1) {
            return ResponseJson.error("状态值只能为0或1");
        }
        authProductMapper.updateProductStatusByProductId(productId, status);
        if (status == 0) {
            return ResponseJson.success("下线授权商品成功");
        } else {
            return ResponseJson.success("上线授权商品成功");
        }
    }

    @Override
    public ResponseJson deleteProduct(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常，请输入授权商品id");
        }
        // 删除商品
        authProductMapper.deleteProductByProductId(productId);
        // 删除商品参数
        authProductMapper.deleteParamsByProductId(productId);
        return ResponseJson.success("删除授权商品成功");
    }

    @Override
    public ResponseJson saveProduct(ProductSaveDto productSaveDto, boolean importFlag) throws IOException {
        Integer productId = productSaveDto.getProductId();
        Integer authId = productSaveDto.getAuthId();
        Integer brandId = productSaveDto.getBrandId();
        Integer productTypeId = productSaveDto.getProductTypeId();
        String productName = productSaveDto.getProductName();
        String productImage = productSaveDto.getProductImage();
        String snCode = productSaveDto.getSnCode();
        String certificateImage = productSaveDto.getCertificateImage();
        Integer addQrCodeFlag = productSaveDto.getAddQrCodeFlag();
        Integer addTemplateType = productSaveDto.getAddTemplateType();
        Integer createBy = productSaveDto.getCreateBy();
        String purchaseWay = productSaveDto.getPurchaseWay();
        String invoiceImage = productSaveDto.getInvoiceImage();
        Integer source = null == productSaveDto.getSource() ? 1 : productSaveDto.getSource();
        List<ProductParamPo> paramList = productSaveDto.getParamList();
        if (null == authId) {
            return ResponseJson.error("参数异常，请输入授权id", null);
        }
        AuthVo auth = authMapper.getAuthById(authId);
        if (null == productTypeId) {
            // 新增设备分类，需要传入设备名称，品牌id，参数列表
            if (null == brandId) {
                return ResponseJson.error("参数异常，请输入品牌id", null);
            }
            if (StringUtils.isEmpty(productName)) {
                return ResponseJson.error("参数异常，请输入设备分类id或设备名称", null);
            }
            if (StringUtils.isNotEmpty(productName)) {
                ProductTypePo productType = authProductMapper.getProductType(null, productName, auth.getAuthUserId());
                if (null != productType) {
                    productTypeId = productType.getProductTypeId();
                }
            }
            if (null == paramList || paramList.size() <= 0) {
                return ResponseJson.error("参数异常，商品参数列表不能为空", null);
            }
        }
        if (StringUtils.isBlank(snCode)) {
            return ResponseJson.error("参数异常，请输入商品SN码", null);
        }
        Integer productIdBySnCode = authProductMapper.getProductIdBySnCode(snCode);
        if (null != productIdBySnCode && !productIdBySnCode.equals(productId)) {
            return ResponseJson.error("参数异常，该商品SN码已存在，请重新输入", null);
        }
        /*if (1 == source && StringUtils.isBlank(certificateImage)) {
            return ResponseJson.error("参数异常，请上传授权牌照", null);
        }*/
        if (!importFlag && 1 == source) {
            if (null == addQrCodeFlag) {
                return ResponseJson.error("参数异常，请选择是否生成二维码授权牌", null);
            }
            if (1 == addQrCodeFlag && null == addTemplateType) {
                return ResponseJson.error("参数异常，请选择二维码授权牌模板", null);
            }
        }
        // 是否为添加操作
        boolean insertFlag = null == productId;
        /*
            组装商品数据
         */
        ProductPo product = new ProductPo();
        // 授权id
        product.setAuthId(authId);
        // 品牌id
        product.setBrandId(brandId);
        // 设备分类id
        product.setProductTypeId(productTypeId);
        if (null == productTypeId) {
            // 设备名称（仅机构添加新设备时使用）
            product.setName(productName);
            // 设备图片（仅机构添加新设备时使用）
            product.setImage(productImage);
        }
        // sn码
        product.setSnCode(snCode);
        // 授权牌照
        product.setCertificateImage(certificateImage);
        product.setAddQrCodeFlag(addQrCodeFlag);
        product.setAddTemplateType(addTemplateType);
        // 购买渠道
        product.setPurchaseWay(purchaseWay);
        // 发票图片
        product.setInvoiceImage(invoiceImage);
        // 商品信息保存后，上线状态默认为“待上线”，审核状态为“待审核”
        /*product.setStatus(2);
        product.setAuditStatus(2);*/
        // 供应商保存，直接上线；机构保存，需要供应商审核通过后才上线
        product.setStatus(1 == source ? 1 : 2);
        product.setAuditStatus(1 == source ? 1 : 2);
        Integer adminUserId = authMapper.getAdminUserId();
        product.setAuditBy(adminUserId);
        product.setAuditTime(new Date());
        product.setShopAuditStatus(1 == source ? 1 : 2);
        if (insertFlag) {
            // 原授权牌照
            product.setOriginalCertificateImage(product.getCertificateImage());
            // 创建人id
            product.setCreateBy(createBy);
            // 创建来源
            product.setCreateSource(source);
            // 创建时间
            product.setCreateTime(new Date());
            if (1 == source && StringUtils.isNotEmpty(product.getCertificateImage()) && (null == addQrCodeFlag || 0 == addQrCodeFlag)) {
                //授权牌照不添加二维码，直接添加水印
                product.setPcCertificateImage(addWaterMark(product.getCertificateImage(), 1));
                product.setAppletsCertificateImage(addWaterMark(product.getCertificateImage(), 2));
            }
            // 插入授权商品
            authProductMapper.insertProduct(product);
            if (1 == source && StringUtils.isNotEmpty(product.getCertificateImage()) && (null != addQrCodeFlag && 1 == addQrCodeFlag)) {
                // 生成二维码授权牌
                String addQrCodeImg = addQrCode(product.getCertificateImage(), product.getProductId(), addTemplateType);
                product.setCertificateImage(addQrCodeImg);
                //授权牌照添加水印
                product.setPcCertificateImage(addWaterMark(addQrCodeImg, 1));
                product.setAppletsCertificateImage(addWaterMark(addQrCodeImg, 2));
                //更新授权牌照
                authProductMapper.updateCertificateImage(product);
            }
            // 自动生成授权牌

        } else {
            // 商品id
            product.setProductId(productId);
            if (1 == source) {
                ProductPo dbProduct = authProductMapper.getImageByProductId(productId);
                // 数据库该商品授权牌是否添加二维码
                Integer dbAddQrCodeFlag = dbProduct.getAddQrCodeFlag();
                // 数据库该商品授权牌添加二维码模板
                Integer dbAddTemplateType = dbProduct.getAddTemplateType();
                // 授权牌照是否重新上传
                boolean updateImageFlag = (StringUtils.isNotBlank(product.getCertificateImage()) && !product.getCertificateImage().equals(dbProduct.getCertificateImage())) || (StringUtils.isEmpty(dbProduct.getPcCertificateImage()) || StringUtils.isEmpty(dbProduct.getAppletsCertificateImage()));
                boolean addQrCodeSign = false;
                boolean addWaterMarkSign = false;
                if (StringUtils.isNotEmpty(product.getCertificateImage())) {
                    if (null != dbAddQrCodeFlag && 1 == dbAddQrCodeFlag) {
                        if (0 == addQrCodeFlag) {
                            // 原本是，现在否
                            if (updateImageFlag) {
                                // 新上传图片，将原图片替换为当前图片，生成水印图片
                                product.setOriginalCertificateImage(product.getCertificateImage());
                                addWaterMarkSign = true;
                            } else {
                                // 原图片，将当前授权牌替换为原图,生成水印图片
                                product.setCertificateImage(dbProduct.getOriginalCertificateImage());
                                addWaterMarkSign = true;
                            }
                        } else if (1 == addQrCodeFlag) {
                            // 原本是，现在是
                            if (updateImageFlag) {
                                // 新上传图片，将原图片替换为当前图片，生成二维码授权牌，生成水印图片
                                product.setOriginalCertificateImage(product.getCertificateImage());
                                addQrCodeSign = true;
                                addWaterMarkSign = true;
                            } else if (!addTemplateType.equals(dbAddTemplateType)) {
                                // 原图片，更改模板，将当前授权牌替换为原图，生成二维码授权牌，生成水印图片
                                product.setCertificateImage(dbProduct.getOriginalCertificateImage());
                                addQrCodeSign = true;
                                addWaterMarkSign = true;
                            }
                            // else 原图片，不更改模板，不做更改
                        }
                    } else {
                        if (1 == addQrCodeFlag) {
                            // 原本否，现在是
                            if (updateImageFlag) {
                                // 新上传图片，将原图片替换为当前图片，生成二维码授权牌，生成水印图片
                                product.setOriginalCertificateImage(product.getCertificateImage());
                                addQrCodeSign = true;
                                addWaterMarkSign = true;
                            } else {
                                // 原图片，生成二维码授权牌，生成水印图片
                                addQrCodeSign = true;
                                addWaterMarkSign = true;
                            }
                        } else {
                            // 原本否，现在否
                            if (updateImageFlag) {
                                // 新上传图片，将原图片替换为当前图片，生成二维码授权牌，生成水印图片
                                product.setOriginalCertificateImage(product.getCertificateImage());
                                addWaterMarkSign = true;
                            }
                            // 原图片，不做更改
                        }
                    }
                }else {
                    product.setPcCertificateImage(null);
                    product.setAppletsCertificateImage(null);
                }
                if (addQrCodeSign) {
                    // 生成二维码授权牌
                    String addQrCodeImg = addQrCode(product.getCertificateImage(), productId, addTemplateType);
                    product.setCertificateImage(addQrCodeImg);
                }
                if (addWaterMarkSign) {
                    //授权牌照添加水印
                    product.setPcCertificateImage(addWaterMark(product.getCertificateImage(), 1));
                    product.setAppletsCertificateImage(addWaterMark(product.getCertificateImage(), 2));
                }
            }
            // 自动生成授权牌

            // 更新授权商品
            authProductMapper.updateProductByProductId(product);
        }
        // 删除商品参数
        authProductMapper.deleteParamsByProductId(product.getProductId());
        // 保存商品参数
        if (null == productTypeId) {
            paramList.forEach(param -> {
                if (StringUtils.isNotBlank(param.getParamName()) && StringUtils.isNotBlank(param.getParamContent())) {
                    authProductMapper.insertProductParam(product.getProductId(), param.getParamName(), param.getParamContent());
                }
            });
        }
        return ResponseJson.success("保存授权商品成功");
    }

    private String addWaterMark(String image,Integer type) throws IOException {
        //type:1pc,2applets
        ClassPathResource classPathResource = new ClassPathResource("/images/newPcWaterMark.png");
        InputStream inputStreamImg = classPathResource.getInputStream();
        Image pcWaterMarkImg = ImageIO.read(inputStreamImg);
        classPathResource = new ClassPathResource("/images/newAppletsWaterMark.png");
        inputStreamImg = classPathResource.getInputStream();
        Image appletsWaterMarkImg = ImageIO.read(inputStreamImg);
        String img = ImageUtils.markImageByIcon(type == 1 ? pcWaterMarkImg : appletsWaterMarkImg, image, null);
        String imagePath = null;
        if (StringUtils.isNotEmpty(img)) {
            MultipartFile imgFile = Base64Util.base64ToMultipart(img);
            if (null != imgFile) {
                imagePath = uploadService.saveFile(imgFile);
            }
            log.info(">>>>>>>>>>>>>>>>水印图片上传成功:" + imagePath);
        } else {
            log.info(">>>>>>>>>>>>>>>>水印图片上传失败");
        }
        return imagePath;
    }

    private String addQrCode(String certificateImage, Integer productId, Integer addTemplateType) throws IOException {
        String qrCodeLink = wwwServer + "/product/auth/product-" + productId + ".html";
        String qrCodeImage = ImageUtils.addQrCode(certificateImage, qrCodeLink, addTemplateType);
        String imagePath = null;
        if (StringUtils.isNotEmpty(qrCodeImage)) {
            MultipartFile imgFile = Base64Util.base64ToMultipart(qrCodeImage);
            if (null != imgFile) {
                imagePath = uploadService.saveFile(imgFile);
            }
            log.info(">>>>>>>>>>>>>>>>二维码图片上传成功:" + imagePath);
        } else {
            log.info(">>>>>>>>>>>>>>>>二维码图片上传失败");
        }
        return imagePath;
    }

    @Override
    public ResponseJson<ProductFormVo> getProductFormData(Integer productId) {
        if (null == productId) {
            return ResponseJson.error("参数异常，请输入商品id", null);
        }
        ProductFormVo productForm = authProductMapper.getProductFormByProductId(productId);
        if (null == productForm) {
            return ResponseJson.error("商品不存在", null);
        }
        List<ProductParamPo> paramList = authProductMapper.getProductTypeParamList(productForm.getProductTypeId());
        if (null == paramList || paramList.size() == 0) {
            paramList = authProductMapper.getParamsByProductId(productId);
        }
        productForm.setParamList(paramList);
        return ResponseJson.success(productForm);
    }

    @Override
    public List<Integer> getProductIdsByAuthId(Integer authId) {
        return authProductMapper.getProductIdsByAuthId(authId);
    }

    @Override
    public ResponseJson updateAllWaterMark() {
        List<ProductPo> productList = authProductMapper.getAllImage();
        productList.forEach(product->{
            try {
                if (StringUtils.isNotEmpty(product.getImage())) {
                    //商品图片添加水印
                    product.setPcImage(addWaterMark(product.getImage(), 1));
                    product.setAppletsImage(addWaterMark(product.getImage(), 2));
                }
                if (StringUtils.isNotEmpty(product.getCertificateImage())) {
                    //授权牌照添加水印
                    product.setPcCertificateImage(addWaterMark(product.getCertificateImage(), 1));
                    product.setAppletsCertificateImage(addWaterMark(product.getCertificateImage(), 2));
                }
                // 更新授权商品
                authProductMapper.updateImageByProductId(product);
            } catch (IOException e) {
                e.printStackTrace();
            }
        });
        return ResponseJson.success("更新商品水印图片成功");
    }

    @Override
    public ResponseJson auditProduct(Integer productId, Integer auditStatus, String invalidReason, Integer auditBy, Integer source) {
        if (productId == null) {
            return ResponseJson.error("请输入商品id");
        }
        if (auditStatus == null) {
            return ResponseJson.error("请输入审核结果");
        }
        if (auditStatus == 0 && StringUtils.isEmpty(invalidReason)) {
            return ResponseJson.error("请输入审核不通过的原因");
        }
        if (auditBy == null) {
            return ResponseJson.error("请输入审核人用户id");
        }
        source = null == source ? 1 : source;
        Date auditTime = new Date();
        int status = 0;
        if (1 == auditStatus) {
            status = 1;
        }
        if (1 == source) {
            // 获取设备信息
            ProductPo product = authProductMapper.getProductPo(productId);
            Integer productTypeId = product.getProductTypeId();
            if (1 == auditStatus && null == productTypeId && StringUtils.isNotEmpty(product.getName()) && StringUtils.isNotEmpty(product.getImage())) {
                // 新增设备分类
                Integer authUserId = authProductMapper.getAuthUserIdByProductId(productId);
                // 查询设备分类是否已存在
                ProductTypePo productType = authProductMapper.getProductType(null, product.getName(), authUserId);
                if (null == productType) {
                    List<ProductParamPo> paramList = authProductMapper.getParamsByProductId(productId);
                    try {
                        ResponseJson<ProductTypePo> result = saveProductType(null, authUserId, product.getBrandId(), product.getName(), product.getImage(), product.getCreateBy(), paramList, 2);
                        int code = result.getCode();
                        if (0 == code) {
                            productType = result.getData();
                            productTypeId = productType.getProductTypeId();
                            // 设备分类直接审核通过
                            auditProductType(productTypeId, 1, null, auditBy);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    productTypeId = productType.getProductTypeId();
                }
            }
            authProductMapper.updateProductAuditStatus(productId, productTypeId, status, auditStatus, invalidReason, auditBy, auditTime);
        }
        if (2 == source) {
            // 获取设备信息
            ProductPo product = authProductMapper.getProductPo(productId);
            Integer productTypeId = product.getProductTypeId();
            if (1 == auditStatus && null == productTypeId && StringUtils.isNotEmpty(product.getName()) && StringUtils.isNotEmpty(product.getImage())) {
                // 新增设备分类
                Integer authUserId = authProductMapper.getAuthUserIdByProductId(productId);
                // 查询设备分类是否已存在
                ProductTypePo productType = authProductMapper.getProductType(null, product.getName(), authUserId);
                if (null == productType) {
                    List<ProductParamPo> paramList = authProductMapper.getParamsByProductId(productId);
                    try {
                        ResponseJson<ProductTypePo> result = saveProductType(null, authUserId, product.getBrandId(), product.getName(), product.getImage(), product.getCreateBy(), paramList, 2);
                        int code = result.getCode();
                        if (0 == code) {
                            productType = result.getData();
                            productTypeId = productType.getProductTypeId();
                            // 设备分类直接审核通过
                            auditProductType(productTypeId, 1, null, auditBy);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                    }
                } else {
                    productTypeId = productType.getProductTypeId();
                }
            }
            authProductMapper.updateProductShopAuditStatus(productId, productTypeId, status, auditStatus, invalidReason, auditBy, auditTime);
        }
        return ResponseJson.success("审核商品成功");
    }

    @Override
    public ResponseJson<PageInfo<WxProductListVo>> getWxProductList(Integer listType, Integer authId, String authParty, Integer productTypeId, String snCode, Integer pageNum, Integer pageSize) {
        listType = null == listType ? 1 : listType;
        if (1 == listType && null == productTypeId) {
            return ResponseJson.error("参数异常，请输入设备分类id", null);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<WxProductListVo> productList = authProductMapper.getWxProductList(listType, authId, authParty, productTypeId, snCode);
        PageInfo<WxProductListVo> pageData = new PageInfo<>(productList);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson<AuthProductVo> getAuthProductDetails(Integer productId) {
        AuthProductVo authProduct = authProductMapper.getAuthProductByProductId(productId);
        if (null == authProduct) {
            return ResponseJson.error("商品不存在", null);
        }
        // 代理声明文件
        if (null != authProduct.getStatementType() && authProduct.getStatementType() == 4) {
            StatementFileVo statementFile = authProductMapper.getStatementFile(authProduct.getAuthUserId(), authProduct.getBrandId());
            OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
            // 设置URL过期时间为1个小时
            Date expiration = new Date(System.currentTimeMillis() + 3600L * 1000);
            String url = ossClient.generatePresignedUrl(bucketName, active + "/authFile/" + statementFile.getOssName(), expiration).toString();
            statementFile.setUrl(url);
            ossClient.shutdown();
            authProduct.setStatementFile(statementFile);
        }
        // 商品参数
        authProduct.setParamList(authProductMapper.getAuthProductParams(productId));
        return ResponseJson.success(authProduct);
    }

    @Override
    public ResponseJson<ProductTypePo> saveProductType(Integer productTypeId, Integer authUserId, Integer brandId, String name, String image, Integer createBy, List<ProductParamPo> paramList, Integer createSource) throws IOException {
        // 是否为添加操作
        boolean insertFlag = null == productTypeId;
        /*
         * 组装设备分类数据
         */
        ProductTypePo productType = new ProductTypePo();
        productType.setAuthUserId(authUserId);
        productType.setBrandId(brandId);
        productType.setName(name);
        productType.setImage(image);
        productType.setCreateBy(createBy);
        productType.setCreateTime(new Date());
        // 上线状态默认为“待上线”，审核状态为“待审核”
        /*productType.setStatus(2);
        productType.setAuditStatus(2);*/
        // 供应商保存，直接上线；机构保存，需要供应商审核通过后才上线
        productType.setStatus(1 == createSource ? 1 : 2);
        productType.setAuditStatus(1 == createSource ? 1 : 2);
        Integer adminUserId = authMapper.getAdminUserId();
        productType.setAuditBy(adminUserId);
        productType.setAuditTime(new Date());
        productType.setCreateSource(createSource);
        if (insertFlag) {
            // 创建人id
            productType.setCreateBy(createBy);
            // 创建时间
            productType.setCreateTime(new Date());
            // 设备分类图片添加水印
            productType.setPcImage(addWaterMark(productType.getImage(),1));
            productType.setAppletsImage(addWaterMark(productType.getImage(),2));
            // 插入设备分类
            authProductMapper.insertProductType(productType);
        } else {
            // 设备分类id
            productType.setProductTypeId(productTypeId);
            ProductTypePo dbProductType = authProductMapper.getProductType(productTypeId, null, null);
            // 设备分类图片是否重新上传
            boolean updateImageFlag = (StringUtils.isNotBlank(productType.getImage()) && !productType.getImage().equals(dbProductType.getImage()))
                    || (StringUtils.isEmpty(productType.getPcImage()) || StringUtils.isEmpty(productType.getAppletsImage()));
            if (updateImageFlag) {
                //商品图片添加水印
                productType.setPcImage(addWaterMark(productType.getImage(), 1));
                productType.setAppletsImage(addWaterMark(productType.getImage(), 2));
            }
            // 更新设备分类
            authProductMapper.updateProductType(productType);
        }
        // 删除设备分类参数
        authProductMapper.deleteParamsByProductTypeId(productType.getProductTypeId());
        // 保存商品参数
        if (null != paramList) {
            paramList.forEach(param -> {
                if (StringUtils.isNotBlank(param.getParamName()) && StringUtils.isNotBlank(param.getParamContent())) {
                    authProductMapper.insertProductTypeParam(productType.getProductTypeId(), param.getParamName(), param.getParamContent());
                }
            });
        }
        return ResponseJson.success("保存设备分类成功", productType);
    }

    @Override
    public ResponseJson deleteProductType(Integer productTypeId) {
        Integer count = authProductMapper.getProductCountByTypeId(productTypeId);
        if (count > 0) {
            return ResponseJson.error("该设备有绑定设备认证，暂时无法删除");
        }
        authProductMapper.deleteProductType(productTypeId);
        authProductMapper.deleteParamsByProductTypeId(productTypeId);
        return ResponseJson.success("删除设备分类成功");
    }

    @Override
    public ResponseJson updateProductTypeStatus(Integer productTypeId, Integer status) {
        authProductMapper.updateProductTypeStatus(productTypeId, status);
        if (status == 0) {
            return ResponseJson.success("下线设备分类成功");
        } else {
            return ResponseJson.success("上线设备分类成功");
        }
    }

    @Override
    public ResponseJson<PageInfo<ProductTypeListVo>> getProductTypeList(Integer listType, Integer authUserId, String name, Integer status, Integer auditStatus, Integer pageNum, Integer pageSize) {
        listType = null == listType ? 1 : listType;
        PageHelper.startPage(pageNum, pageSize);
        List<ProductTypeListVo> productList = authProductMapper.getProductTypeList(listType, authUserId, name, status, auditStatus);
        PageInfo<ProductTypeListVo> pageData = new PageInfo<>(productList);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson<List<ProductTypeListVo>> getProductTypeSelectList(Integer authUserId) {
        List<ProductTypeListVo> productTypeList = authProductMapper.getProductTypeList(null, authUserId, null, 1, 1);
        return ResponseJson.success(productTypeList);
    }

    @Override
    public ResponseJson auditProductType(Integer productTypeId, Integer auditStatus, String invalidReason, Integer auditBy) {
        Date auditTime = new Date();
        int status = 0;
        if (1 == auditStatus) {
            status = 1;
        }
        authProductMapper.updateProductTypeAuditStatus(productTypeId, status, auditStatus, invalidReason, auditBy, auditTime);
        return ResponseJson.success("审核设备分类成功");
    }

    @Override
    public ResponseJson<PageInfo<WxProductTypeListVo>> getWxProductTypeList(Integer authUserId, String appId, String name, Integer pageNum, Integer pageSize) {
        if (null == authUserId && StringUtils.isNotEmpty(appId)) {
            authUserId = shopMapper.getUserIdByAppId(appId);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<WxProductTypeListVo> productTypeList = authProductMapper.getWxProductTypeList(authUserId, name);
        PageInfo<WxProductTypeListVo> pageData = new PageInfo<>(productTypeList);
        return ResponseJson.success(pageData);
    }

    @Override
    public ResponseJson<ProductTypePo> getProductTypeFromData(Integer productTypeId) {
        ProductTypePo productType = authProductMapper.getProductType(productTypeId, null, null);
        List<ProductParamPo> paramList = authProductMapper.getProductTypeParamList(productTypeId);
        productType.setParamList(paramList);
        return ResponseJson.success(productType);
    }

}
